var ZhLoggerDeviceFile = (function() {
	var registerTag = function(dataSourceName, tag) {
		if (window.ZhNativeInterface) {
			tag = JSON.stringify(tag);
			var fname = dataSourceName + "[tags].dat";
			var file = ZhNativeInterface.readFile(fname);
			if (file) {
				var all = file.split("\n");
				all.pop();
				if (all.indexOf(tag) === -1) {
					ZhNativeInterface.writeToFile(tag + "\n", fname);
				}
			} else {
				ZhNativeInterface.writeToFile(tag + "\n", fname);
			}
		}
	};
	var getFilesTags = function(dataSourceName) {
		if (window.ZhNativeInterface) {
			var fname = dataSourceName + "[tags].dat";
			var file = ZhNativeInterface.readFile(fname);
			var dataSet = [];
			if (file) {
				var all = file.split("\n");
				all.pop();
				dataSet.push(
					all.map(function(log) {
						return JSON.parse(log);
					})
				);
			}
			return dataSet;
		}
	};

	function ZhLoggerDeviceFile() {

	}

	ZhLoggerDeviceFile.prototype.save = function(dataSourceName, logObj) {
		if (window.ZhNativeInterface) {
			if (logObj.tag) {
				registerTag(dataSourceName, logObj.tag);
			}
			var fname = dataSourceName + "_" + logObj.mode + (!!logObj.tag ? ("_" + logObj.tag) : "") + ZhLoggerDeviceFile.LOGFILE_EXTENSION;
			ZhNativeInterface.writeToFile(JSON.stringify(logObj) + '\n', fname);
		}
	};

	ZhLoggerDeviceFile.prototype.find = function(dataSourceName, mode, tag, date) {
		if (window.ZhNativeInterface) {
			var logFiles = [];
			var file;
			var tags = getFilesTags(dataSourceName);
			if (mode) {
				if (!tag) {
					file = ZhNativeInterface.readFile(dataSourceName + "_" + mode + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
					if (file) {
						logFiles.push(file);
					}
					tags.forEach(function(tag) {
						file = ZhNativeInterface.readFile(dataSourceName + "_" + mode + "_" + tag + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
						if (file) {
							logFiles.push(file);
						}
					});
				} else {
					file = ZhNativeInterface.readFile(dataSourceName + "_" + mode + "_" + tag + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
					if (file) {
						logFiles.push(file);
					}
				}
			} else {
				var modes = [ZhLogger.DEBUG, ZhLogger.INFO, ZhLogger.WARNING, ZhLogger.ERROR, ZhLogger.FATAL];
				modes.forEach(function(mode) {
					if (!tag) {
						tags.forEach(function(tag) {
							file = ZhNativeInterface.readFile(dataSourceName + "_" + mode + "_" + tag + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
							if (file) {
								logFiles.push(file);
							}
						});
						file = ZhNativeInterface.readFile(dataSourceName + "_" + mode + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
						if (file) {
							logFiles.push(file);
						}
					} else {
						file = ZhNativeInterface.readFile(dataSourceName + "_" + mode + "_" + tag + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
						if (file) {
							logFiles.push(file);
						}
					}
				});
			}
			var dataSet = [];
			var parse = function(log) {
				return JSON.parse(log);
			};
			var dateFltr = function(log) {
				return (!date || (date && log.date > date));
			};
			logFiles.forEach(function(logFile) {
				if (logFile) {
					var all = logFile.split("\n");
					all.pop();
					all = all.map(parse);
					all = all.filter(dateFltr);
					if (all.length) {
						dataSet.push(all);
					}
				}
			});
			return dataSet;
		}
	};

	ZhLoggerDeviceFile.prototype.clear = function(dataSourceName, mode) {
		if (window.ZhNativeInterface) {
			var fnames = [];
			var tags = getFilesTags(dataSourceName);
			var modes = [ZhLogger.DEBUG, ZhLogger.INFO, ZhLogger.WARNING, ZhLogger.ERROR, ZhLogger.FATAL];
			if (mode) {
				tags.forEach(function(tag) {
					ZhNativeInterface.removeFile(dataSourceName + "_" + mode + "_" + tag + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
				});
				modes.forEach(function(mode) {
					ZhNativeInterface.removeFile(dataSourceName + "_" + mode + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
				});
			} else {
				modes.forEach(function(mode) {
					tags.forEach(function(tag) {
						ZhNativeInterface.removeFile(dataSourceName + "_" + mode + "_" + tag + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
					});
					ZhNativeInterface.removeFile(dataSourceName + "_" + mode + ZhLoggerDeviceFile.LOGFILE_EXTENSION);
					ZhNativeInterface.removeFile(dataSourceName + "[tags].dat");
				});
			}
		}
	};
	return ZhLoggerDeviceFile;
})();
ZhLoggerDeviceFile.LOGFILE_EXTENSION = ".log";